<?php 

/*
 * form.php
 * Copies a given object to create a new one.
 *
 * Vars that come in as POST vars
 *  - source_dn (rawurlencoded)
 *  - new_dn (form element)
 *  - server_id
 */

require 'config.php';
require_once 'functions.php';

$source_dn = stripslashes( rawurldecode( $_POST['old_dn'] ) );
$dest_dn = stripslashes( $_POST['new_dn'] );
$encoded_dn = rawurlencode( $old_dn );
$source_server_id = $_POST['server_id'];
$dest_server_id = $_POST['dest_server_id'];
$do_recursive = $_POST['recursive'] == 'on' ? true : false;

check_server_id( $source_server_id ) or die( "Bad server_id: " . htmlentities( $source_server_id ) );
have_auth_info( $source_server_id ) or die( "Not enough information to login to server. Please check your configuration." );
check_server_id( $dest_server_id ) or die( "Bad server_id: " . htmlentities( $dest_server_id ) );
have_auth_info( $dest_server_id ) or die( "Not enough information to login to server. Please check your configuration." );

/* Error checking */
if( 0 == strlen( trim( $dest_dn ) ) )
{
	echo "Error: You left the destination DN blank.";
	exit;
}

if( strcasecmp( $source_dn,$dest_dn ) == 0 && $source_server_id == $dest_server_id )
{
	echo "Error: the source and destination DN is the same.";
	exit;
}

if( $do_recursive ) {
	// build a tree similar to that of the tree browser to give to r_copy_dn
	$tree = array();
	build_tree( $source_server_id, $source_dn, $tree );
	$copy_result = r_copy_dn( $source_server_id, $dest_server_id, $tree, $source_dn, $dest_dn );
} else {
	$copy_result = copy_dn( $source_server_id, $source_dn, $dest_server_id, $dest_dn );
}

if( $copy_result )
{
	$edit_url="edit.php?server_id=$dest_server_id&dn=" . rawurlencode( $dest_dn );
	$new_rdn = get_rdn( $dest_dn );
	$container = get_container( $dest_dn );
	// update the session tree to reflect the change
	session_start();
	if( session_is_registered( 'tree' ) )
	{
		$tree = $_SESSION['tree'];
		if( isset( $tree[$dest_server_id][$container] ) )
		{
			$tree[$dest_server_id][$container][] = $dest_dn;
			$_SESSION['tree'] = $tree;
			session_write_close();
		}
	}

	?>
		<html>
		<head>
		<!-- refresh the tree view (with the new DN renamed)
		and redirect to the edit_dn page -->
		<script language="javascript">
			parent.left_frame.location.reload();
		</script>


		<meta http-equiv="refresh" content="0; url=<?php echo $edit_url; ?>" />
		</head>
		<body>
		<script language="javascript">
			parent.left_frame.location.reload();
		</script>
		Redirecting... <a href="<?php echo $edit_url; ?>">here</a>.
		</body>
		</html>
		<?php 
}
else
{
	echo "Error: could not create destination object.<br />";
	echo "LDAP said: " . ldap_error( $ds ) . ' 0x' . dechex( ldap_errno( $ds ) );
	exit;
}

function r_copy_dn( $source_server_id, $dest_server_id, &$tree, $root_dn, $dest_dn )
{
	$copy_result = copy_dn( $source_server_id, $root_dn, $dest_server_id, $dest_dn );
	
	if( ! $copy_result ) {
		global $R_COPY_ERROR;
		$R_COPY_ERROR = "Failed to copy $root_dn (server: $source_server_id) to $dest_dn (server: $dest_server_id)\n";
		echo $R_COPY_ERROR. "\n";
		return false;
	}
	
	$children = $tree[ $root_dn ];
	if( is_array( $children ) && count( $children ) > 0 )
	{
		foreach( $children as $child_dn ) {
			$child_rdn = get_rdn( $child_dn );
			$new_dest_dn = $child_rdn . ',' . $dest_dn;
			r_copy_dn( $source_server_id, $dest_server_id, $tree, $child_dn, $new_dest_dn );
		}
	}
	else
	{
		return true;
	}

	return true;
}

function copy_dn( $source_server_id, $source_dn, $dest_server_id, $dest_dn )
{
	global $ds;
	$ds = davedap_ldap_connect( $dest_server_id ) or die( "Could not connect to LDAP server" );
	$attrs = get_object_attrs( $source_server_id, $source_dn );
	$new_entry = $attrs;
	unset( $new_entry['dn'] );
	$add_result = @ldap_add( $ds, $dest_dn, $new_entry );
	return $add_result;
}

function build_tree( $source_server_id, $root_dn, &$tree )
{
	$children = get_container_contents( $source_server_id, $root_dn );
	if( is_array( $children ) && count( $children ) > 0 )
	{
		$tree[ $root_dn ] = $children;
		foreach( $children as $child_dn )
			build_tree( $source_server_id, $child_dn, $tree );
	}

}
