<?php 

/*
 * create_form.php
 * Displays a last chance confirmation form to delete a dn.
 *
 * Variables that come in as GET vars or POST vars:
 *  - server_id
 *  - container (rawurlencoded)
 *  - step (the step number, unset the first time, set the second time)
 *  - object_classes (comes in as a serialized, rawurlencoded string of objectclasses (Array))
 */

require 'config.php';
require_once 'functions.php';

$server_id = $_REQUEST['server_id'];
$step = $_REQUEST['step'] ? $_REQUEST['step'] : 1; // defaults to 1
$container = stripslashes( $_REQUEST['container'] );
$server_name = $servers[$server_id]['name'];
$oclasses = $_REQUEST['object_classes'];
$rdn = stripslashes( $_REQUEST['rdn'] );

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );

?>

<?php include 'header.php'; ?>
<body>

<h3 class="title"><img src="images/star.png" /> Create Object </h3>
<h3 class="subtitle">Step <?php echo $step; ?> of 2</h3>

<?php  if( $step == 1 ) { ?>

	<?php  

		$oclasses = get_schema_objectclasses( $server_id );
		if( count( $oclasses ) == 0 )
		{
			echo "Error: Could not get schema for server: $server_id";
			exit;
		}

	?>

	<form action="create_form.php" method="post">
	<input type="hidden" name="step" value="2" />
	<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
	<table class="create">
	<tr>
		<td>RDN:</td>
		<td><input type="text" name="rdn" value="" size="20" /> (i.e.: cn=MyNewObject)</td>
	</tr>
	<tr>
		<td>Container:</td>
		<td><input type="text" name="container" size="40" value="<?php echo htmlentities($container); ?>" /></td>
	</tr>
	<tr>
		<td>ObjectClass(es):<br /><br />('top' will be<br />included<br />automagically)<br /> </td>
		<td><select name="object_classes[]" multiple>
			<?php  foreach( $oclasses as $oclass => $attrs ) { ?>
	
				<option value="<?php echo htmlentities($oclass); ?>">
					<?php echo htmlentities($attrs['name']); ?>
				</option>
	
			<?php  } ?>
			</select><br /><i>ctrl+click to select multiple</i>
		</td>
	</table>

	<center>
	<br />
	<input type="submit" name="submit" value="Proceed to Step 2 >>" />
	</form>

<?php  } elseif( $step == 2 ) {

	if( strlen( $rdn ) == 0 )
	{
		echo "Error: You left the RDN field blank";
		exit;
	}

	if( strlen( $container ) == 0 )
	{	
		echo "Error: You left the container field blank";
		exit;
	}

	if( count( $oclasses ) == 0 )
	{
		echo "Error: You must specify at least one objectClass from the list";
		exit;
	}

	// build a list of required attributes:
	$dn = $rdn . ',' . $container;
	$schema = get_schema( $server_id );
	$required_attrs = array();
	$all_attrs = array();
	foreach( $oclasses as $oclass ) {
		$required_attrs = array_merge( $required_attrs, $schema['oclasses'][strtolower($oclass)]['must_attrs'] );
		$all_attrs = array_merge( $all_attrs, $schema['oclasses'][strtolower($oclass)]['must_attrs'],
					  $schema['oclasses'][strtolower($oclass)]['may_attrs'] );
	}
	$required_attrs = array_unique( $required_attrs );
	$all_attrs = array_unique( $all_attrs );
	sort( $required_attrs );
	sort( $all_attrs );
	$attr_select_html = "";
	foreach( $all_attrs as $a )
		$attr_select_html .= "<option>" . htmlentities($a) . "</option>\n";

	?>

	Specify attributes and values for the new object:<br />
	<b><?php echo htmlentities($dn); ?></b><br /><br />

	<small><b>Instrucions</b>: Enter the values for the <?php echo count($required_attrs); ?>
	required attributes in the form below. Then, enter any optional attributes. You
	can specify multiple values for one attribute by selecting that attribute
	multiple times in the "Optional Attributes" section.</small><br />
	<br />

	<form action="create.php" method="post">
	<input type="hidden" name="step" value="2" />
	<input type="hidden" name="new_dn" value="<?php echo htmlentities( $dn ); ?>" />
	<input type="hidden" name="new_rdn" value="<?php echo htmlentities( $rdn ); ?>" />
	<input type="hidden" name="container" value="<?php echo htmlentities( $container ); ?>" />
	<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
	<input type="hidden" name="object_classes" value="<?php echo rawurlencode(serialize($oclasses)); ?>" />

	<table class="edit_dn" cellspacing="0">
	<tr><th colspan="2">Required Attributes</th></tr>
	<?php  foreach( $required_attrs as $count => $attr ) { ?>
		<?php  if( $count % 2 == 0 ) { ?>
			<tr class="row1">
		<?php  } else { ?>
			<tr class="row2">
		<?php  } ?>
		<td class="attr"><b><?php echo htmlentities($attr); ?></b></td>
		<td class="val"><input type="text" name="required_attrs[<?php echo htmlentities($attr); ?>]" value="" size="40" />
	</tr>
	<?php  } ?>

	<tr><th colspan="2">Optional Attributes</th></tr>

	<?php  for($i=0; $i<10; $i++ ) { ?>
		<?php  if( $i % 2 == 0 ) { ?>
			<tr class="row1">
		<?php  } else { ?>
			<tr class="row2">
		<?php  } ?>
		<td class="attr"><select name="attrs[<?php echo $i; ?>]"><?php echo $attr_select_html; ?></select></td>
		<td class="val"><input type="text" name="vals[<?php echo $i; ?>]" value="" size="40" />
	</tr>
	<?php  } ?>
	</table>

	<center>
	<input type="submit" name="submit" value="Create Object" />
	</center>

<?php  } else { ?>

	Hmm, unrecognized step value: <?php echo htmlentities($step); ?>

<?php  } ?>

</body>
</html>
