<?php 

/*
 * edit.php
 * Displays the specified dn from the specified server for editing
 *
 * Variables that come in as GET vars:
 *  - dn (rawurlencoded)
 *  - server_id
 */

require 'config.php';
require_once 'functions.php';

$dn = stripslashes( rawurldecode( $_GET['dn'] ) );
$encoded_dn = rawurlencode( $dn );
$updated_attr = stripslashes( $_GET['updated_attr'] );
$server_id = $_GET['server_id'];
$rdn = ldap_explode_dn( $dn, 0 );
$rdn = $rdn[0];

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );

$attrs = get_object_attrs( $server_id, $dn );

if( ! $attrs )
{
	echo "Error: No such dn, " . htmlentities( utf8_decode( $dn ) );
	exit;
}

$server_name = $servers[$server_id]['name'];

?>

<?php include 'header.php'; ?>
<body>

<h3 class="title"><?php echo htmlentities( utf8_decode( $rdn ) ); ?></h3>
<h3 class="subtitle">Server: <b><?php echo $server_name; ?></b> &nbsp;&nbsp;&nbsp; Distinguished Name: <b><?php echo htmlentities( utf8_decode( $dn ) ); ?></b></h3>

<table class="edit_dn_menu">

<?php if( 0 != strcasecmp( $dn, $servers[$server_id]['base'] ) ) { ?>
<?php /* We won't allow them to delete the base dn of the server */ ?>
<tr>
	<td><img src="images/trash.png" /></td>
	<td><a href="delete_form.php?server_id=<?php echo $server_id; ?>&amp;dn=<?php echo $encoded_dn; ?>"
	       title="You will be prompted to confirm this decision">Delete this object</a></td>
</tr> 
<?php } ?>

<tr>
	<td><img src="images/cut.png" /></td>
	<td><a href="copy_form.php?server_id=<?php echo $server_id; ?>&amp;dn=<?php echo $encoded_dn?>"
	     title="Copy this object to another location, a new DN, or another server">Copy this object</a></td>
</tr> 
<tr>
	<td><img src="images/save.png" /></td>
	<?php $ldif_url = "ldif_export.php?server_id=$server_id&amp;dn=$encoded_dn&amp;scope=base"; ?>
	<td><a href="<?php echo $ldif_url; ?>" title="Save an LDIF dump of this object">Export to LDIF</a> 
		(<a href="<?php echo $ldif_url; ?>&amp;format=mac" title="Macintosh style carriage returns">mac</a>)
		(<a href="<?php echo $ldif_url; ?>&amp;format=win" title="Windows style carriage returns">win</a>)
		(<a href="<?php echo $ldif_url; ?>&amp;format=unix" title="Unix style carriage returns">unix</a>)
	</td>
</tr>
<tr>
	<td><img src="images/star.png" /></td>
	<td><a href="<?php echo "create_form.php?server_id=$server_id&amp;container=$encoded_dn"; ?>">Create a child object</a></td>
</tr>

<?php flush(); ?>
<?php $children = get_container_contents( $server_id, $dn ); if( ($children_count = count( $children ) ) > 0 ) { ?>

<tr>
	<td><img src="images/children.png" /></td>
	<td><a href="search.php?search=true&amp;server_id=<?php echo $server_id; ?>&amp;filter=<?php echo rawurlencode('objectClass=*'); ?>&amp;base_dn=<?php echo $encoded_dn; ?>&amp;form=advanced&amp;scope=one">View <?php echo $children_count; ?> <?php echo ($children_count==1?'child':'children');?></a></td>
</tr>

<?php } ?>

<?php if( $children_count > 0 ) { ?>

<tr>
	<td><img src="images/save.png" /></td>
	<?php $ldif_url = "ldif_export.php?server_id=$server_id&amp;dn=$encoded_dn&amp;scope=sub"; ?>
	<td><a href="<?php echo $ldif_url; ?>" title="Save an LDIF dump of this object and all of its children">Export subtree to LDIF</a> 
		(<a href="<?php echo $ldif_url; ?>&amp;format=mac" title="Macintosh style carriage returns">mac</a>)
		(<a href="<?php echo $ldif_url; ?>&amp;format=win" title="Windows style carriage returns">win</a>)
		(<a href="<?php echo $ldif_url; ?>&amp;format=unix" title="Unix style carriage returns">unix</a>)
	</td>
</tr>

<?php } ?>

</table>
<br />

<table class="edit_dn" cellspacing="0">

<tr class="row1">
<td class="attr"><b>rdn</b></td>
<td class="val">
	<nobr>
	<form action="rename.php" method="post" class="edit_dn" />
	<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
	<input type="hidden" name="dn" value="<?php echo $encoded_dn; ?>" />
	<input type="text" name="new_rdn" size="40" value="<?php echo htmlentities( utf8_decode( $rdn ) ); ?>" />
	<input class="update_dn" type="submit" value="Rename" />
	</form>
	</nobr>
</td>	
<tr><td colspan="2"></td></tr>

<?php if( $edit_dn_schema_lookup ) $schema_attrs = get_schema_attributes( $server_id ); ?>
<?php  foreach( $attrs as $attr => $vals ) { ?>

	<?php  if( $attr == 'dn' ) continue; ?>

	<?php  if( $attr == $updated_attr ) { ?>
	
		<tr class="updated_attr">
	
	<?php  } else { ?>
		<?php  if( $counter++ % 2 == 0 ) { ?>
	
			<tr class="row2">
	
		<?php  } else { ?>
	
			<tr class="row1">

		<?php  } ?>
	<?php  } ?>

	<?php $add_href = "add_value_form.php?server_id=$server_id&amp;dn=$encoded_dn&amp;attr=" . rawurlencode( $attr ); ?>

	<td class="attr"><b><?php echo $attr; ?></b><br /><small>(<a href="<?php echo $add_href; ?>"
								     title="Add an additional value to this attribute">add value</a>)</small>
	</td>

	<td class="val">

	<?php	/*
		 * This is next IF statement is a KLUGE!! If anyone knows a better way to check for
		 * binary data that works with UTF-8 encoded strings, please help
		 */
	?>

	<?php if( 0==strcasecmp( $attr, 'jpegPhoto' ) || 0==strcasecmp( $attr, 'networkAddress' ) ) { ?>

		<small>This attribute contains binary data,<br />which cannot be safely displayed<br />or edited in a web-browser.</small></td>
		<?php continue; ?>

	<?php } ?>

	<?php /* is this a  multi-valued attribute? */ ?>
	<?php  if( is_array( $vals ) ) { ?>
		<?php  foreach( $vals as $val ) { ?>

			<?php $val = utf8_decode( $val ); ?>

			<nobr>
			<form action="update.php" method="post" class="edit_dn" />
			<input type="hidden" name="multiple_values" value="true" />
			<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
			<input type="hidden" name="attr" value="<?php echo rawurlencode($attr); ?>" />
			<input type="hidden" name="dn" value="<?php echo rawurlencode($dn); ?>" />
			<input type="hidden" name="old_value" value="<?php echo htmlentities($val); ?>" />
			<input type="text" name="new_value" size="40" value="<?php echo htmlentities($val); ?>" />
			<input class="update_dn" type="submit" value="Update" />
			</form>
			</nobr>

		<?php  } ?>
	<?php  } else { ?>
		<?php  $val = $vals; ?>

		<?php $val = utf8_decode( $val ); ?>

		<nobr>
		<form action="update.php" method="post" class="edit_dn" />
		<input type="hidden" name="attr" value="<?php echo rawurlencode($attr); ?>" />
		<input type="hidden" name="dn" value="<?php echo rawurlencode($dn); ?>" />
		<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
		<input type="hidden" name="old_value" value="<?php echo htmlentities($val); ?>" />

		<?php  if( 0 == strcasecmp( $attr, 'userpassword' ) ) { ?>

			<?php  preg_match( "/{([^}]+)}/", $val, $enc_type); $enc_type = $enc_type[1]; ?>
			<input size="30" type="text" name="new_value" value="<?php echo htmlentities($val); ?>" />
			<select name="enc_type">
				<option>clear</option>
				<option<?php echo $enc_type=='crypt'?' selected':''; ?>>crypt</option>
				<option<?php echo $enc_type=='md5'?' selected':''; ?>>md5</option>
				<option<?php echo $enc_type=='sha'?' selected':''; ?>>sha</option>
			</select>

		<?php } elseif( $edit_dn_schema_lookup && 
				'Boolean' == $schema_attrs[ strtolower($attr) ]['type'] ) { ?>

			<select name="new_value">
				<option value="TRUE"<?php echo ($val=='TRUE' ?  ' selected' : ''); ?>>TRUE</option>
				<option value="FALSE"<?php echo ($val=='FALSE' ? ' selected' : ''); ?>>FALSE</option>
			</select>

		<?php  } else { ?>

			<input size="40" type="text" name="new_value" value="<?php echo htmlentities($val); ?>" />

		<?php  } ?>
		<input class="update_dn" type="submit" value="Update" />
		</form>
		</nobr>

	<?php  } ?>

	</td>
	</tr>

<?php  } ?>

<?php 

// build a list of attributes available for this object based on its objectClasses
$oclasses = $attrs['objectclass'];
if( ! is_array( $oclasses ) )
	$oclasses = array( $oclasses );
$avail_attrs = array();
$schema_oclasses = get_schema_objectclasses( $server_id, true );
foreach( $oclasses as $oclass )
	$avail_attrs = array_merge( $schema_oclasses[ strtolower( $oclass ) ]['must_attrs'],
				    $schema_oclasses[ strtolower( $oclass ) ]['may_attrs'],
				    $avail_attrs );

$avail_attrs = array_unique( $avail_attrs );
$avail_attrs = array_filter( $avail_attrs, "not_an_attr" );
 
sort( $avail_attrs );

if( $counter % 2 == 0 )
	echo '<tr class="row2">';
else
	echo '<tr class="row1">';

?>

<form action="new_attr.php" method="post">
<input type="hidden" name="server_id" value="<?php echo $server_id; ?>" />
<input type="hidden" name="dn" value="<?php echo $encoded_dn; ?>" />

<td class="attr"><nobr><img src="images/star.png" width="18" height="18" align="left" /><b>New Attribute</b></nobr></td>
<td class="val"><nobr>
<select name="attr">
<?php  foreach( $avail_attrs as $a ) { ?>
	<option value="<?php echo htmlentities($a); ?>"><?php echo htmlentities($a); ?></option>
<?php  } ?>
</select>
<input type="text" name="val" size="20" />
<input type="submit" name="submit" value="Add" class="update_dn" />
</nobr></td>
</form>
</tr>

</table>


<?php 

function not_an_attr( $x )
{
	global $attrs;
	return ! isset( $attrs[ strtolower( $x ) ] );
}

?>
