<?php 

/*
 * ldif_export.php
 * Dumps the LDIF file for a given DN
 *
 * Variables that come in as GET vars:
 *  - dn (rawurlencoded)
 *  - server_id
 *  - format (one of 'win', 'unix', 'mac'
 *  - scope (one of 'sub', 'base', or 'one')
 */

require 'config.php';
require_once 'functions.php';

$dn = stripslashes( rawurldecode( $_GET['dn'] ) );
$server_id = $_GET['server_id'];
$format = $_GET['format'];
$scope = $_GET['scope'] ? $_GET['scope'] : 'base';

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );

$objects = davedap_ldap_search( $server_id, 'objectClass=*', $dn, array(), $scope, false );

//echo "<pre>";
//print_r( $objects );
//exit;

$rdn = ldap_explode_dn( $dn, 0 );
$rdn = $rdn[0];

switch( $format ) {
	case 'win': 	$br = "\r\n"; break;
	case 'mac': 	$br = "\r"; break;
	case 'unix': 
	default:	$br = "\n"; break;
}
		
if( ! $objects )
{
	echo "Search on dn (" . htmlentities($dn) . ") came back empty";
	exit;
}

header( "Content-type: text/plain" );
header( "Content-disposition: attachment; filename=$rdn.ldif" );
header( "Expires: Mon, 26 Jul 1997 05:00:00 GMT" ); 
header( "Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT" ); 
header( "Cache-Control: no-store, no-cache, must-revalidate" );
header( "Cache-Control: post-check=0, pre-check=0", false );
header( "Pragma: no-cache" );

echo "version: 1$br$br";
echo "# LDIF Export for: $rdn$br";
echo "# Scope: $scope, " . count( $objects ) . " objects$br";
echo "# Generated by DaveDAP on " . date("F j, Y g:i a") . "$br";
echo $br;

foreach( $objects as $dn => $attrs )
{
	unset( $attrs['dn'] );
	unset( $attrs['count'] );

	echo "dn: $dn\n";
	foreach( $attrs as $attr => $val )
		if( is_array( $val ) )
			foreach( $val as $v )
				echo "$attr: $v$br";
		else
			echo "$attr: $val$br";
	echo $br;
}
