<?php 

/*
 * rename.php
 * Renames a DN to a different name. 
 *
 * Variables that come in as POST vars:
 *  - dn (rawurlencoded)
 *  - server_id
 *  - new_rdn
 */

require 'config.php';
require_once 'functions.php';

$dn = rawurldecode( stripslashes( $_POST['dn'] ) );
$server_id = $_POST['server_id'];
$new_rdn = stripslashes( $_POST['new_rdn'] );
$new_rdn = utf8_encode($new_rdn);

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );

$ds = davedap_ldap_connect( $server_id ) or die( "Could not connect to LDAP sever" );
 
// build the container string
$old_rdn = ldap_explode_dn( $dn, 0 );
$container = $old_rdn[ 1 ];
for( $i=2; $i<count($old_rdn)-1; $i++ )
	$container .= ',' . $old_rdn[$i];

if( ! $container )
{
	echo "Error: Container is null!<br />";
	exit;
}

if( ! ldap_rename( $ds, $dn, $new_rdn, $container, false ) )
{
	echo "Error: Could not rename the object.<br />";
	echo "LDAP said: \"" . ldap_error( $ds ) . "\"<br />";
	if( 0 == strcasecmp( ldap_error($ds), "Not Supported" ) )
	{
		echo "This usually means that we could not get into LDAP version 3 mode. ";
		echo "If your LDAP server doesn't support LDAP version 3, then you won't ";
		echo "be able to rename objects like this.";
	}
	exit;
}
else
{
	// update the session tree to reflect the change
	session_start();
	if( session_is_registered( 'tree' ) )
	{
		// search and destroy
		foreach( $tree[$server_id] as $tree_dn => $subtree )
			foreach( $subtree as $key => $sub_tree_dn )
				if( 0 == strcasecmp( $sub_tree_dn, $dn ) ) 
					$tree[$server_id][$tree_dn][$key] = $new_rdn . ',' . $container;

		session_write_close();

		$edit_url="edit.php?server_id=$server_id&dn=" . rawurlencode( "$new_rdn,$container" );

		?>

		<html>
		<head>
			<!-- refresh the tree view (with the new DN renamed)
			     and redirect to the edit_dn page -->
			<script language="javascript">
			parent.left_frame.location.reload();
			location.href='<?php echo $edit_url; ?>';
			</script>

			<!-- If the JavaScript didn't work, here's a meta tag to the job -->
			<meta http-equiv="refresh" content="0; url=<?php echo $edit_url; ?>" />
		</head>
		<body>
		Redirecting... click <a href="<?php echo $edit_url; ?>">here</a> if you're impatient.
		</body>
		</html>

		<?php 

	}
}
