<?php 

/* 
 * schema.php
 * Displays the schema for the specified server_id
 * 
 * Variables that come in as GET vars:
 *  - server_id
 */

require 'config.php';
require_once 'functions.php';

$server_id = $_GET['server_id'];
$schema = get_schema( $server_id );

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );
davedap_ldap_connect( $server_id ) or die( "Coult not connect to LDAP server." );
?>

<?php include 'header.php'; ?>
<body>

<?php  if( ! $schema ) { ?>

	Could not retrieve schema from <b> <?php echo $servers[$server_id]['name']; ?></b>.<br />
	<br />
	This could happen for several reasons, the most probable of which are:
	<ul>
		<li>The server does not fully support the LDAP protocol.</li>
		<li>Your version of PHP does not correctly perform the query.</li>
	</ul>

<?php  } else { ?>

	<h3 class="title">Schema for server <b><?php echo $servers[$server_id]['name']; ?></b></h3>
	<h3 class="subtitle"><?php echo count($schema['oclasses']); ?> objectClasses</h3>

	<?php  flush(); ?>
	
	<small>Jump to an objectClass:</small>
	<select name="oclass_jumper"
		onChange="window.location.href='schema.php?server_id=<?php echo $server_id; ?>#'+this.value">
	<?php  foreach( $schema['oclasses'] as $oclass => $desc ) {  ?>

		<option value="<?php echo $oclass; ?>"><?php echo $desc['name']; ?></option>

	<?php  } ?>
	</select>
	<br />

	<?php  foreach( $schema['oclasses'] as $oclass => $attrs ) { ?>
		
		<small>[<a name="<?php echo $oclass; ?>" href="#" title="Head on up to the top.">top</a>]</small>
		<h4 class="oclass">objectClass <b><?php echo $attrs['name']; ?></b></h4>
		<h4 class="oclass_sub">OID <b><?php echo $attrs['oid']; ?></b></h4>
		<?php if( $attrs['description'] ) { ?>
		<h4 class="oclass_sub">Description <b><?php echo $attrs['description']; ?></b></h4>
		<?php } ?>
		<h4 class="oclass_sub">Inherits <b><?php echo $attrs['sup']; ?></b></h4>
		<br />

		<b>Required Attrbibutes</b>
		<ul class="schema">
		<?php  if( count( $attrs['must_attrs'] ) > 0 ) { ?>
			<?php  foreach( $attrs['must_attrs'] as $attr ) { ?>
				<li>
					<?php echo $attr; ?>
					<ul>
						<?php if( is_array( $schema['attrs'][strtolower($attr)] ) )
							foreach( $schema['attrs'][strtolower($attr)] as $key => $val )
								if( $key != 'val' && $val != null ) { ?>
									<li><small><?php echo $key; ?>: <?php echo $val; ?></small></li>
								<?php  } ?>
					</ul>
				</li>
			<?php  } ?>
		<?php  } else { ?>
			(none)
		<?php  } ?>
		</ul>
		<b>Optional Attrbibutes</b>
		<ul class="schema">
		<?php  if( count( $attrs['may_attrs'] ) > 0 ) { ?>
			<?php  foreach( $attrs['may_attrs'] as $attr ) { ?>
				<li>
					<?php echo $attr; ?>
					<ul>
						<?php if( is_array( $schema['attrs'][strtolower($attr)] ) )
							foreach( $schema['attrs'][strtolower($attr)] as $key => $val )
								if( $key != 'val' && $val != null ) { ?>
									<li><small><?php echo $key; ?>: <?php echo $val; ?></small></li>
								<?php  } ?>
					</ul>
				</li>
			<?php  } ?>
		<?php  } else { ?>
			(none)
		<?php  } ?>
		</ul>
	<?php  } ?>

<?php  } ?>

</body>

</html>

