<?php 

/*
 * search.php
 * Perform LDAP searches and draw the advanced/simple search forms
 *
 * Variables that come in as GET vars:
 *  - server_id
 *  - search (true if performing a search, empty to just draw form)
 *  For simple searches:
 *  - attribute, criterion, filter
 *  For advanced searches:
 *  - base_dn, scope, filter
 */

require 'config.php';
require_once 'functions.php';

process_config();

$server_id = $_GET['server_id'];

// try to get an available server_id if one is not provided
if( ! isset( $server_id ) )
{
	$server_id = get_avail_server_id();
}
else
{
	check_server_id( $server_id ) or die( "Bad server_id: " . var_dump( htmlentities( $server_id ) ) );
}

// build the server drop-down html and JavaScript array (for base_dns)
$server_menu_html = '<select name="server_id" onChange="base_dn.value = server_base_dns[ this.value ]">';
$js_dn_list = '';
foreach( $servers as $id => $server ) { 
	$js_dn_list .= '"' . $server['base'] . '",';
	if( $server['host'] ) { 
		$server_menu_html .= '<option value="'.$id.'"' . ( $id==$server_id? ' selected' : '' ) . '>';
		$server_menu_html .= $server['name'] . '</option>';
	}
}
// trim off the trailing comma
$js_dn_list = substr( $js_dn_list, 0, strlen($js_dn_list)-1 );
$server_menu_html .= '</select>';

$filter = stripslashes( $_GET['filter'] );
$filter = utf8_encode($filter);
$attr = stripslashes( $_GET['attribute'] );
$base_dn = $_GET['base_dn'] ? $_GET['base_dn'] : $servers[$server_id]['base'];
$criterion = stripslashes( $_GET['criterion'] );
$form = stripslashes( $_GET['form'] );
$scope = $_GET['scope'] ? $_GET['scope'] : 'sub';
//echo "<PRE>";print_r( $_GET );echo "</pre>";
?>

<?php include 'header.php'; ?>
<body>

<center>

<?php  if( $form == 'advanced' ) { ?>

	<form action="search.php" method="get" class="search">
	<input type="hidden" name="search" value="true" />
	<input type="hidden" name="form" value="advanced" />
	
	<center><b>Advanced Search Form</b></center>
	<small>(<a href="search.php?server_id=<?php echo $server_id; ?>&amp;form=simple">Simple Search Form</a>)</small><br />
	<br />

	<table>
		<tr>
			<td><small>Server</small></td>
			<td><?php echo $server_menu_html; ?></td>
		</tr>
	
		<tr>
			<td><small>Base DN</small></td>
			<td><input type="text" name="base_dn" value="<?php echo htmlentities($base_dn); ?>" size="30" id="base_dn" /></td>
		</tr>
	
		<tr>
			<td><small>Search Scope</small></td>
			<td>
				<select name="scope">
				<option<?php echo $scope=='sub'?' selected':''; ?>>sub</option>
				<option<?php echo $scope=='one'?' selected':''; ?>>one</option>
				<option<?php echo $scope=='base'?' selected':''; ?>>base</option>
				</select>
			</td>

		</tr>

		<tr>
			<td><small>Search Filter</small></td>
			<td><input type="text" name="filter" size="30" value="<?php echo htmlentities(utf8_decode($filter)); ?>" /></td>
		</tr>
	
		<tr>
			<td colspan="2"><br /><center><input type="submit" value="Search" /></center></td>
		</tr>
	</table>
	</form>


<?php  } else { ?>

	<form action="search.php" method="get" class="search">
	<input type="hidden" name="search" value="true" />
	<input type="hidden" name="form" value="simple" />
	<input type="hidden" name="scope" value="sub" />

	<table>
	<td>	
	<center><b>Simple Search Form</b><br />
	<small>(<a href="search.php?server_id=<?php echo $server_id; ?>&amp;form=advanced">Advanced Search Form</a>)</small><br />
	<br />
	</center>
	
			<small>Server</small><br /> <?php echo $server_menu_html; ?><br />
			<br />	
			<small>Search for entries whose:</small><br />

			<nobr>
			<select name="attribute">
			<?php  foreach( $search_attributes as $id => $attribute ) { ?>
				<option value="<?php echo rawurlencode( $attribute ); ?>"<?php echo $attribute==$attr?' selected':''; ?>>
					<?php echo htmlentities($search_attributes_display[$id]); ?>
				</option>
			<?php  } ?>
			</select>

			<select name="criterion">
			<?php  foreach( $search_criteria_options as $c ) { ?>
				<option value="<?php echo $c; ?>"<?php echo $c==$criterion?' selected':''; ?>>
					<?php echo htmlentities($c); ?>
				</option>
			<?php  } ?>
			</select>
				
			<input type="text" name="filter" size="20" value="<?php echo htmlentities(utf8_decode($filter)); ?>" /><br />
			<br />
	
			<center><input type="submit" value="Search" /></center>
			</nobr>
	</td>
	</table>
	</form>

<?php  } ?>

</center>

<?php flush(); ?>

<?php 

if( $_GET['search'] )
{
	// do we have enough authentication information for the specified server_id
	if( ! have_auth_info( $server_id ) )
	{
		$login_url = "login_form.php?server_id=$server_id&amp;redirect=" . rawurlencode( $_SERVER['REQUEST_URI'] );
		?>
		<center>
		<br />
		You haven't logged into server <b><?php echo htmlentities( $servers[$server_id]['name'] ); ?></b>
		yet. Go to the <a href="<?php echo $login_url; ?>">login form</a> to do so.
		</center>
		<?php
		exit;
	}

	davedap_ldap_connect( $server_id ) or die( "<center>Could not connect to LDAP server.</center>" );
	
	if( $filter )
	{

		// if they are using the simple search form, build an LDAP search filter from their input
		if( $form == 'simple' )
		{
			switch( $criterion ) {
				case 'starts with':
					$filter = "($attr=$filter*)";
					break;
				case 'contains':
					$filter = "($attr=*$filter*)";
					break;
				case 'ends with':
					$filter = "($attr=*$filter)";
					break;
				case 'equals':
					$filter = "($attr=$filter)";
					break;
				case 'sounds like':
					$filter = "($attr~=$filter)";
					break;
				default:
					echo "Unrecognized criteria option: " . htmlentities( $criterion );
					echo "If you want to add your own criteria to the list. Be sure to edit ";
					echo "search.php to handle them. Quitting.";
					exit;
			}
		}
		
		$time_start = utime();
		$results = davedap_ldap_search( $server_id, $filter, $base_dn,
				array_merge( $search_result_attributes, array( $search_result_title_attribute ) ),
				$scope );
		$time_end = utime();
		$time_elapsed = round( $time_end - $time_start, 2 );
		$count = count( $results );
		?>

		<br />
		<center>Found <b><?php echo $count; ?></b> <?php echo $count==1?'entry':'entries'; ?>.

		<?php  if( $form == 'simple' ) { ?>
			<center><small>Filter performed: <?php echo htmlentities( $filter ); ?></small></center>
		<?php  } ?>

		</center>

		<?php flush(); ?>	

		<?php if( $results ) foreach( $results as $dn => $attrs ) { ?>
			<?php  $encoded_dn = rawurlencode($attrs['dn']); ?>
			<?php  $rdn = utf8_decode( get_rdn( $attrs['dn'] ) ); ?>
			<div class="search_result"><a href="edit.php?server_id=<?php echo $server_id; ?>&amp;dn=<?php echo $encoded_dn; ?>"><?php echo htmlentities($rdn); ?></a></div>
			<table class="attrs">
				<?php  foreach( $search_result_attributes as $attr ) { ?>

					<tr>
						<td class="attr" valign="top"><?php echo htmlentities($attr); ?></td>
						<td class="val">
							<?php  if( is_array( $attrs[strtolower($attr)] ) ) { ?>
								<?php  foreach( $attrs[strtolower($attr)] as $a ) { ?>
									<?php echo str_replace( ' ', '&nbsp;', htmlentities(utf8_decode($a))); ?><br />
								<?php  } ?>
							<?php  } else { ?>
								<?php echo str_replace( ' ', '&nbsp;', htmlentities(utf8_decode($attrs[strtolower($attr)]))); ?>
							<?php  } ?>
						</td>
					</tr>

				<?php  } ?>
			</table>

		<?php  } ?>

			<br /><br />
			<div class="search_result"><center><span style="font-weight:normal;font-size:75%;">Search happily performed by DaveDAP in 
				<b><?php echo $time_elapsed; ?></b> seconds.</small></center></div>
		<?php 
	}
}

?>

</body>
</html>

<?php 

function utime ()
{
	$time = explode( " ", microtime());
	$usec = (double)$time[0];
	$sec = (double)$time[1];
	return $sec + $usec;
}

?>
