<?php 

/* 
 * tree.php
 * This script displays the LDAP tree for all the servers that you have
 * in config.php. We read the session variable 'tree' to know which
 * dns are expanded or collapsed.
 */

require 'config.php';
require_once 'functions.php';

// no expire header stuff
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
header("Pragma: no-cache");

// do we not have a tree yet? Build a new one.
session_start();
if( ! session_is_registered( 'tree' ) ) {
	session_register( 'tree' );
	$_SESSION['tree'] = build_initial_tree(); 
}

// grab the tree out of the session variable
$tree = $_SESSION['tree'];
?>

<?php include 'header.php'; ?>
<body>
	<table class="tree" cellspacing="0">

	<?php 
	foreach( $servers as $server_id => $server_tree ) { 
		if( $servers[$server_id]['host'] != '' ) { 
			draw_tree_html( $servers[$server_id]['base'], $server_id );
		}
	}
	?>

	</table>
	<?php  /*echo "<pre>"; print_r( $tree ); echo "</pre>";*/ ?>
</body>
</html>

<?php 

// store our tree in the session and finish up
$_SESSION['tree'] = $tree;
session_write_close();
exit;

/*
 * Recursively descend on the given dn and draw the tree in html
 */
function draw_tree_html( $dn, $server_id, $level=0 )
{
	global $servers, $tree;
	$id = $server_id;
	
	$encoded_dn = rawurlencode( $dn );
	$expand_href = "expand.php?server_id=$id&amp;dn=$encoded_dn";
	$collapse_href = "collapse.php?server_id=$id&amp;dn=$encoded_dn";
	$edit_href = "edit.php?server_id=$id&amp;dn=$encoded_dn";
	$img_src = 'images/' . get_image_name( $dn );

	// draw root of server (base_dn)
	if( $level == 0 )
	{
		$schema_href = 'schema.php?server_id=' . $server_id  . '" target="right_frame';
		$search_href= 'search.php?server_id=' . $server_id  . '" target="right_frame';
		echo '<tr class="server">';
		echo '<td class="icon"><img src="images/server.png" alt="server"/></td>';
		echo '<td colspan="99"><b><a name="' . $id . '">' . $servers[$id]['name'] . '</a></b><br />';

		// do we have what it takes to authenticate here, or do we need to
		// present the user with a form (for 'form' auth_types).
		if( have_auth_info( $server_id ) )
		{
			echo '<small>(<a href="' . $schema_href . '">schema</a> | ';
			echo '<a href="' . $search_href . '">search</a> | ';
			echo '<a href="refresh.php?server_id=' . $server_id . '">refresh</a>';
			echo ')';
			echo '</td></tr>';
			$rdn = utf8_decode( $dn );
		}
		else
		{
			$login_href = "login_form.php?server_id=$server_id";
			echo '&nbsp;&nbsp;&nbsp;<a href="' . $login_href . '" target="right_frame">';
			echo '<img src="images/uid.png" align="top" alt="login"/></a> ';
			echo '<a href="' . $login_href . '" target="right_frame">login...</a>';
			echo '</td></tr>';
			return;
		}
	}
	else
	{
		$rdn = ldap_explode_dn( $dn, 0 );
		$rdn = utf8_decode( $rdn[0] );
	}

	echo '<tr>';

	for( $i=0; $i<=$level; $i++ ) {
		echo '<td class="spacer"></td>';
	}
		
	// is this node expanded?
	if( isset( $tree[$server_id][$dn] ) ) { ?>
		<td class="expander">
			<nobr>
			<a href="<?php echo $collapse_href; ?>"><img src="images/minus.png" alt="plus" /></a>
			</nobr>
		</td>
		<?php  $object_count = ' <span class="count">(' . count( $tree[$server_id][$dn] ) . ')</span>'; ?>
	<?php  } else { ?>	
		<td class="expander">
			<nobr>
			<a href="<?php echo $expand_href; ?>"><img src="images/plus.png" alt="minus" /></a>
			</nobr>
		</td>
		<?php  $object_count = ''; ?>
	<?php  } ?>	

	<td class="icon">
		<a href="<?php echo $edit_href; ?>"
		   target="right_frame"
		   name="<?php echo $server_id; ?>_<?php echo $encoded_dn; ?>"><img src="<?php echo $img_src; ?>" /></a>
	</td>
	<td class="rdn" colspan="99">
		<nobr><a href="<?php echo $edit_href; ?>" target="right_frame"><?php echo htmlentities($rdn); ?></a><?php echo $object_count; ?></nobr>
	</td>
	</tr>

	<?php 

	if( isset( $tree[$server_id][$dn] ) && is_array( $tree[$server_id][$dn] ) )	{
		foreach( $tree[$server_id][$dn] as $dn ) { 
			draw_tree_html( $dn, $server_id, $level+1 );
		}

		// print the "Create New object" link.
		$create_href = "create_form.php?server_id=$server_id&amp;container=$encoded_dn";
		echo '<tr>';
		for( $i=0; $i<=$level; $i++ ) {
			echo '<td class="spacer"></td>';
		}
		echo '<td class="spacer"></td>';
		echo '<td class="icon"><a href="' . $create_href . '" target="right_frame"><img src="images/star.png" /></a></td>';
		echo '<td class="create" colspan="99"><a href="' . $create_href . '" target="right_frame" title="Create a new object in '. $rdn.'">Create New</a></td></tr>';
	}

	echo '</tr>';

}


?>
