<?php 

/* 
 *  update.php
 *  Updates or deletes a value from a specified 
 *  attribute for a specified dn.
 *  Variables that come in on the query string:
 *  - dn (rawurlencoded)
 *  - attr (rawurlencoded) the attribute we are modifying
 *  - server_id
 *  - old_value (hidden form element) for comparison to the
 * 	current stored value, AND to know which value in a 
 *	multi-valued attribute to update/delete.
 *  - new_value (form element)
 *  - enc_type (optional: only for userPassword attribute)
 *	can be one of: md5, crypt, or clear.
 *  - multiple_values: true/false to tell us if the attr being
 *	updated has multi-values.
 *
 * On success, redirect to the edit_dn page.
 * On failure, echo an error.
 */

require 'config.php';
require_once 'functions.php';

$server_id = $_POST['server_id'];
$dn = stripslashes( rawurldecode( $_POST['dn'] ) );
$encoded_dn = rawurlencode( $dn );
$new_value = stripslashes( $_POST['new_value'] );
$new_value = utf8_encode( $new_value );
$old_value = stripslashes( $_POST['old_value'] );
$old_value = utf8_encode( $old_value );
$attr = stripslashes( $_POST['attr'] );
$encoded_attr = rawurlencode( $attr );
$current_value = get_object_attr( $server_id, $dn, $attr );
$enc_type = $_POST['enc_type'];
$is_multi_value = $_POST['multiple_values'];

check_server_id( $server_id ) or die( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or die( "Not enough information to login to server. Please check your configuration." );

// if this is a user password, check the encryption type to possibly hash the password
// (replacing $new_value with the hashed version)
if( 0 == strcasecmp( $attr, 'userpassword' ) )
{
	switch( $enc_type )
	{
		case 'crypt':
			$new_value = '{crypt}' . crypt( $new_value, random_salt(8) );
			break;
		case 'md5':
			$new_value = '{md5}' . md5( $new_value );
			break;
		case 'sha':
			if( function_exists( 'mhash' ) ) {
				$new_value = '{sha}' . base64_encode( mhash( MHASH_SHA1, $new_value ) );
			} else {
				echo "Error: Your PHP install does not have the mhash() function. Cannot do SHA hashes.";
				echo "Password value unchanged.";
				exit;
			}
			break;
		case 'clear':
		default:
			$new_value = $new_value;
	}

	if( $new_value === false )
	{
		echo "Error: Could not compute the new password hash based on enc_type: " . htmlentities( $enc_type );
		exit;
	}
}

// Does this attribute have more than one entry? If so, we have to
// determine which of the attributes are changing
if( $is_multi_value )
{

	$current_values = $current_value;

	// If the value has changed since they loaded the page and clicked
	// the 'Update' button, then we reject the request.
	if( ! in_array( $old_value, $current_values ) )
	{
		// die
		echo "It appears that the value for '$attr' has changed since you clicked the button. ";
		echo "This could be because someone else has edited that value. You should go back and ";
		echo "try again.";
		exit;
	}
	
	$new_values = array( $attr => $current_values );
	foreach( $new_values[$attr] as $id => $val )
	{
		if( $val == $old_value )
		{
			if( $new_value == '' ) {
				unset($new_values[$attr][$id]);
			} else {
				$new_values[$attr][$id] = $new_value;
			}
			break;
		}
	}

	// For some strange reason, PHP requires that your indices be numerically contiguous when
 	// performing ldap_modify(). Otherwise, it will do some really weird stuff.
	$new_values[$attr] = array_values( $new_values[$attr] );


	$ds = davedap_ldap_connect( $server_id ) or die( "Could not connect to LDAP server" );
	$mod_result = @ldap_modify( $ds, $dn, $new_values );
	if( ! $mod_result )
	{
		echo "Could not perform ldap_modify<br />";
		echo "LDAP said: " . ldap_error( $ds );
		exit;
	}

	header( "Location: edit.php?server_id=$server_id&dn=" .
			rawurlencode( $dn ) . "&updated_attr=" . rawurlencode( $attr ) );
}
// this attribute only has a single value.
else
{
	// is this a password change?
	if( $attr == 'userpassword' )
	{
		$pass_type = $_POST['pass_type'];
	}

	// If the value has changed since they loaded the page and clicked
	// the 'Update' button, then we reject the request.
	if( $old_value != $current_value )
	{
		// die
		echo "It appears that the value for '$attr' has changed since you clicked the button. ";
		echo "This could be because someone else has edited that value. You should go back and ";
		echo "try again.";
		exit;
	}

	$ds = davedap_ldap_connect( $server_id ) or die( "Could not connect to LDAP server" );

	// Is this a request to delete the value?
	if( $new_value == '' )
	{
		// ldap_mod_del
		$new_entry = array();
		$new_entry[$attr] = $old_value;
		$del_result = @ldap_mod_del( $ds, $dn, $new_entry );
		
		if( ! $del_result )
			die( "Could not perform ldap_mod_del operation on attribute '$attr'.<br /><br />".
			     "LDAP said: " . ldap_error( $ds ) );

		header( "Location: edit.php?server_id=$server_id&dn=" .
			 rawurlencode( $dn ) . "&deleted_attr=" . rawurlencode( $attr ) );
	}
	else
	{
		// ldap_mod_replace
		$new_entry = array();
		$new_entry[$attr] = $new_value;
		$mod_result = @ldap_modify( $ds, $dn, $new_entry );

		if( ! $mod_result )
			die( "Could not perform ldap_modify operation on attribute '" . htmlentities($attr) . "'.<br /><br />".
			     "LDAP said: " . ldap_error( $ds ) );

		header( "Location: edit.php?server_id=$server_id&dn=" .
			 rawurlencode( $dn ) . "&updated_attr=" . rawurlencode( $attr ) );

	}
}

; ?>
